@@ -157,9 +157,9 @@ class Softmax(Layer):
         self.outputs = probabilities
         return self.outputs
     
-    def backward(self, dE):
-        dA = np.empty_like(dE)
-        for index, (single_output, single_dvalues) in enumerate(zip(self.outputs, dE)):
+    def backward(self, dX, lr, momentum):
+        dA = np.empty_like(dX)
+        for index, (single_output, single_dvalues) in enumerate(zip(self.outputs, dX)):
             single_output = single_output.reshape(-1, 1)
             jacobian_matrix = np.diagflat(single_output) - np.dot(single_output, single_output.T)
             dA[index] = np.dot(jacobian_matrix,
@@ -240,14 +240,10 @@ class Sequential():
             raise RuntimeError("model not trained")
             
     def backward(self, lr, momentum):
-        if isinstance(self.train_layers[-1], Softmax):
-            self.dX = self.train_layers[-1].backward(self.train_cost_fn.derivative(self.train_targets, self.train_outputs))
-        else:
-            self.dX = self.train_cost_fn.derivative(self.train_targets, self.train_outputs) * self.train_layers[-1].derivative()
+        self.dX = self.train_cost_fn.derivative(self.train_targets, self.train_outputs)
         self.train_layers.reverse()
-        for i in range(len(self.train_layers)):
-            if i > 0:
-                self.dX = self.train_layers[i].backward(self.dX, lr, momentum)
+        for layer in self.train_layers:
+                self.dX = layer.backward(self.dX, lr, momentum)
         self.train_layers.reverse()
         if self.val_inputs is not None and self.val_targets is not None:
             for i in range(len(self.val_layers)):
@@ -257,6 +253,10 @@ class Sequential():
             
     def accuracy(self, targets, outputs):
         accuracy = 1e-10
+        outputs = np.array(outputs)
+        if outputs.shape[1] > 1:
+            for i in range(outputs.shape[0]):
+                outputs[i][np.argmax(outputs[i], axis=0)] = 1.0
         for i in range(outputs.shape[1]):
             if i+2 > outputs.shape[1]:
                 accuracy = np.array(np.sum(np.around(outputs[:,i]) == targets[:,i]) / len(targets[:,i]))
